=== modified file 'README.txt'
--- README.txt	2008-06-16 09:48:20 +0000
+++ README.txt	2008-06-16 00:28:36 +0000
@@ -52,12 +52,15 @@
 locking to prevent corruption.
 
 
-FILES CHANGED CACHE
--------------------
-
-To speed up the display of the changelog view for large trees,
-loggerhead can be configured to cache the files changes between
-revisions.  Set the 'cachepath' value in the config file.
+CACHES
+------
+
+To speed up operations with large branches, loggerhead can be configured
+to create a files changed cache per branch on request:
+
+    - a cache of files changed in a revision
+
+You can put the cache folder anywhere.
 
 
 SUPPORT

=== modified file 'loggerhead.conf.example'
--- loggerhead.conf.example	2008-06-16 09:48:20 +0000
+++ loggerhead.conf.example	2008-06-16 00:28:36 +0000
@@ -27,7 +27,7 @@
     # for the top browsing page, you can include a description, in raw HTML.
     #description = 'Bazaar is a <i>revision control system</i>.'
 
-    # should we cache files changed? (recommended for large trees)
+    # should we cache files changed? (recommended)
     # this will be a folder containing one file.  if the folder doesn't
     # exist, it'll be created.
     #

=== modified file 'loggerhead/branchview.py'
--- loggerhead/branchview.py	2008-06-16 09:26:05 +0000
+++ loggerhead/branchview.py	2008-06-16 00:28:36 +0000
@@ -58,6 +58,7 @@
         # branch history
         self._history_lock = threading.RLock()
         self._history = None
+        self._closed = False
 
         self.changes = ChangeLogUI(self)
         self.revision = RevisionUI(self)
@@ -130,6 +131,8 @@
         calls.  but if the bazaar branch on-disk has been updated since this
         History was created, a new object will be created and returned.
         """
+        if self._closed:
+            return None
         if (self._history is None) or self._history.out_of_date():
             self.log.debug('Reload branch history...')
             _history = self._history = History.from_folder(
@@ -142,6 +145,8 @@
                 _history.use_file_cache(FileChangeCache(_history, cache_path))
         return self._history
 
+        h = self.get_history()
+
     def url(self, elements, **kw):
         "build an url relative to this branch"
         if not isinstance(elements, list):

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2008-06-16 09:51:29 +0000
+++ loggerhead/changecache.py	2008-06-16 00:28:36 +0000
@@ -27,6 +27,7 @@
 
 import cPickle
 import os
+import time
 
 from loggerhead import util
 from loggerhead.lockfile import LockFile
@@ -49,6 +50,8 @@
                 ).replace('?', _param_marker)
 _insert_stmt = ("insert into revisiondata (revid, data) "
                 "values (?, ?)").replace('?', _param_marker)
+_update_stmt = ("update revisiondata set data = ? where revid = ?"
+                ).replace('?', _param_marker)
 
 
 
@@ -77,11 +80,53 @@
             return None
         else:
             return self._unserialize(filechange[0])
-    def add(self, revid_obj_pairs):
+    def add(self, revid_obj_pairs, commit=True):
         for  (r, d) in revid_obj_pairs:
             self.cursor.execute(_insert_stmt, (r, self._serialize(d)))
-        self.connection.commit()
-
+        if commit:
+            self.connection.commit()
+    def update(self, revid_obj_pairs, commit=True):
+        for  (r, d) in revid_obj_pairs:
+            self.cursor.execute(_update_stmt, (self._serialize(d), r))
+        if commit:
+            self.connection.commit()
+    def count(self):
+        self.cursor.execute(
+            "select count(*) from revisiondata")
+        return self.cursor.fetchone()[0]
+    def close(self, commit=False):
+        if commit:
+            self.connection.commit()
+        self.connection.close()
+
+        self.log.info('Building revision cache...')
+        start_time = time.time()
+        last_update = time.time()
+        count = 0
+
+        work = list(self.history.get_revision_history())
+        jump = 100
+        for i in xrange(0, len(work), jump):
+            r = work[i:i + jump]
+            # must call into history so we grab the branch lock (otherwise, lock inversion)
+            self.history.get_changes(r)
+            if self.closed():
+                self.flush()
+                return
+            count += jump
+            now = time.time()
+            if now - start_time > max_time:
+                self.log.info('Cache rebuilding will pause for now.')
+                self.flush()
+                return
+            if now - last_update > 60:
+                self.log.info('Revision cache rebuilding continues: %d/%d' % (min(count, len(work)), len(work)))
+                last_update = time.time()
+                self.flush()
+            # give someone else a chance at the lock
+            time.sleep(1)
+        self.log.info('Revision cache rebuild completed.')
+        self.flush()
 
 class FileChangeCache(object):
     def __init__(self, history, cache_path):

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2008-06-16 09:28:18 +0000
+++ loggerhead/controllers/__init__.py	2008-06-16 00:28:36 +0000
@@ -116,3 +116,10 @@
         }
         vals.update(templatefunctions)
         return vals
+
+
+# for use in profiling the very-slow get_change() method:
+#h = Root.bazaar.bzr_dev.get_history()
+#w = list(h.get_revision_history())
+#h._get_changes_profiled(w[:100])
+

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2008-06-16 08:49:28 +0000
+++ loggerhead/controllers/changelog_ui.py	2008-06-16 00:28:36 +0000
@@ -57,7 +57,6 @@
             try:
                 revid, start_revid, revid_list = h.get_view(
                     revid, start_revid, filter_file_id, query)
-                kw['start_revid'] = start_revid
                 util.set_context(kw)
 
                 if (query is not None) and (len(revid_list) == 0):
@@ -102,7 +101,7 @@
                         break
 
             def url(pathargs, **kw):
-                return self._branch.url(pathargs, **util.get_context(**kw))
+                return self._branch.url(pathargs, **kw)
 
             vals = {
                 'branch': self._branch,

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2008-06-16 09:48:20 +0000
+++ loggerhead/history.py	2008-06-18 04:27:23 +0000
@@ -35,6 +35,7 @@
 import time
 from StringIO import StringIO
 
+from loggerhead import search
 from loggerhead import util
 from loggerhead.util import decorator
 
@@ -48,7 +49,6 @@
 import bzrlib.tsort
 import bzrlib.ui
 
-
 with_branch_lock = util.with_lock('_lock', 'branch')
 
 
@@ -352,6 +352,20 @@
         return revid_list[index:]
 
     @with_branch_lock
+    def get_revision_history_matching(self, revid_list, text):
+        self.log.debug('searching %d revisions for %r', len(revid_list), text)
+        z = time.time()
+        # this is going to be painfully slow. :(
+        out = []
+        text = text.lower()
+        for revid in revid_list:
+            change = self.get_changes([ revid ])[0]
+            if text in change.comment.lower():
+                out.append(revid)
+        self.log.debug('searched %d revisions for %r in %r secs', len(revid_list), text, time.time() - z)
+        return out
+
+    @with_branch_lock
     def get_search_revid_list(self, query, revid_list):
         """
         given a "quick-search" query, try a few obvious possible meanings:
@@ -476,12 +490,14 @@
         else:
             revid_list = None
 
-        revid_list = self.get_search_revid_list(query, revid_list)
-        if revid_list and len(revid_list) > 0:
-            if revid not in revid_list:
-                revid = revid_list[0]
-            return revid, start_revid, revid_list
-        else:
+        try:
+            revid_list = search.search_revisions(self._branch, query)
+            if len(revid_list) > 0:
+                if revid not in revid_list:
+                    revid = revid_list[0]
+                return revid, start_revid, revid_list
+        except:
+            # no results
             return None, None, []
 
     @with_branch_lock
@@ -503,6 +519,7 @@
             path = '/' + path
         return self._branch.repository.get_revision_inventory(revid).path2id(path)
 
+
     def get_merge_point_list(self, revid):
         """
         Return the list of revids that have merged this node.
@@ -603,9 +620,21 @@
 
         return changes
 
+    # alright, let's profile this sucka. (FIXME remove this eventually...)
+    def _get_changes_profiled(self, revid_list):
+        from loggerhead.lsprof import profile
+        import cPickle
+        ret, stats = profile(self.get_changes_uncached, revid_list)
+        stats.sort()
+        stats.freeze()
+        cPickle.dump(stats, open('lsprof.stats', 'w'), 2)
+        self.log.info('lsprof complete!')
+        return ret
+
     @with_branch_lock
     @with_bzrlib_read_lock
     def get_changes_uncached(self, revid_list):
+        # FIXME: deprecated method in getting a null revision
         revid_list = filter(lambda revid: not bzrlib.revision.is_null(revid),
                             revid_list)
         parent_map = self._branch.repository.get_graph().get_parent_map(revid_list)

=== added file 'loggerhead/search.py'
--- loggerhead/search.py	1970-01-01 00:00:00 +0000
+++ loggerhead/search.py	2008-06-15 23:10:35 +0000
@@ -0,0 +1,40 @@
+# Copyright (C) 2008  Martin Albisetti <argentina@gmail.com>
+# Copyright (C) 2008  Robert Collins
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+import sets
+import os
+from bzrlib.plugins.search import errors
+from bzrlib.plugins.search import index as _mod_index
+from bzrlib.plugins.search.index import FileTextHit, RevisionHit
+from bzrlib.transport import get_transport
+from bzrlib.plugin import load_plugins
+load_plugins()
+
+def search_revisions(branch, query_list=[]):
+    index = _mod_index.open_index_branch(branch)
+    query = [(query_item,) for query_item in query_list]
+    revid_list = []
+    for result in index.search(query):
+        if isinstance(result, FileTextHit):
+            revid_list.append(result.text_key[1])
+        elif isinstance(result, RevisionHit):
+            revid_list.append(result.revision_key)
+
+    if len(revid_list) == 0:
+        raise errors.NoMatch(query_listo)
+
+    return list(sets.Set(revid_list))

=== modified file 'loggerhead/templates/annotate.pt'
--- loggerhead/templates/annotate.pt	2008-05-29 04:43:39 +0000
+++ loggerhead/templates/annotate.pt	2008-06-12 03:35:23 +0000
@@ -26,7 +26,7 @@
       <div class="links">
         <div>
           <b>&#8594;</b>
-          <a tal:attributes="href python:url(['/files', revid], clear=1)">
+          <a tal:attributes="href python:url(['/files', revno], clear=1)">
             browse files
           </a>
         </div>

=== modified file 'loggerhead/templates/atom.pt'
--- loggerhead/templates/atom.pt	2008-06-10 04:48:26 +0000
+++ loggerhead/templates/atom.pt	2008-06-11 04:41:08 +0000
@@ -14,7 +14,7 @@
     <updated tal:content="entry/date/isoformat">
     Z
     </updated>
-    <id tal:content="python:branch.url(['revision', entry.revid])">
+    <id tal:content="python:branch.url(['revision', entry.revno])">
       ID
     </id>
     <author>
@@ -26,6 +26,6 @@
       comment
     </content>
     <link rel="alternate"
-          tal:attributes="href python:branch.url(['revision', entry.revid])" />
+          tal:attributes="href python:branch.url(['revision', entry.revno])" />
   </entry>
 </feed>

=== modified file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	2008-06-11 04:51:53 +0000
+++ loggerhead/templates/changelog.pt	2008-06-12 03:08:14 +0000
@@ -77,8 +77,8 @@
             <a tal:attributes="name string:entry-${entry/revno}" />
             <tr tal:attributes="class string:revision-header parity${entry/parity}">
               <td class="revision-number">
-                <a tal:attributes="title python:'Show revision '+history.get_revno(entry.revid);
-                                   href  python:url(['/revision', entry.revid])"
+                <a tal:attributes="title python:'Show revision '+entry.revno;
+                                   href  python:url(['/revision', entry.revno], clear=1)"
                    tal:content="python:util.trunc(entry.revno)">
                   text
                 </a>
@@ -87,8 +87,8 @@
                   tal:content="structure python:collapse_button('cl', entry.revno)">
               </td>
               <td class="summary">
-                <a tal:attributes="title python:'Show revision '+history.get_revno(entry.revid);
-                                   href  python:url(['/revision', entry.revid])"
+                <a tal:attributes="title python:'Show revision '+entry.revno;
+                                   href  python:url(['/revision', entry.revno], clear=1)"
                    tal:content="entry/short_comment">
                   text
                 </a>
@@ -101,7 +101,7 @@
                         tal:content="python:util._approximatedate(entry.date)"></span>
               </td>
               <td class="inventory-link">
-                <a tal:attributes="href python:branch.url(['/files', entry.revid]);
+                <a tal:attributes="href python:branch.url(['/files', entry.revno]);
                                    title string:Files at revision ${entry/revno}">
                   files
                 </a>
@@ -121,7 +121,7 @@
                     </th>
                     <td class="children">
                       <a title="Show history"
-                         tal:attributes="href python:url(['/changes'], start_revid=merge_point.revid)"
+                         tal:attributes="href python:url(['/changes'], start_revid=merge_point.revno)"
                          tal:content="python:revno_with_nick(merge_point)">
                         class="revlink">
                         ${text}
@@ -136,7 +136,7 @@
                     </th>
                     <td class="parents">
                       <a title="Show history"
-                         tal:attributes="href python:url(['/changes'], start_revid=parent.revid)"
+                         tal:attributes="href python:url(['/changes'], start_revid=parent.revno)"
                          tal:content="python:revno_with_nick(parent)">
                         class="revlink">
                         ${text}

=== modified file 'loggerhead/templates/inventory.pt'
--- loggerhead/templates/inventory.pt	2008-06-11 04:53:47 +0000
+++ loggerhead/templates/inventory.pt	2008-06-13 05:00:06 +0000
@@ -20,10 +20,10 @@
 
     <div metal:fill-slot="content">
       <div class="links">
-        <div> <b>&#8594;</b> <a tal:attributes="href python:url(['/revision'], clear=1, start_revid=revid)">
+        <div> <b>&#8594;</b> <a tal:attributes="href python:url(['/revision', change.revno])">
 	    view revision </a>
         </div>
-        <div> <b>&#8594;</b> <a tal:attributes="href python:url(['/changes'], clear=1, start_revid=revid)">
+        <div> <b>&#8594;</b> <a tal:attributes="href python:url(['/changes', change.revno])">
 	    view branch changes </a>
         </div>
       </div>
@@ -81,7 +81,7 @@
           </td>
           <td tal:attributes="class string:filename ${file/kind}">
             <a tal:condition="python:file.kind=='directory'"
-               tal:attributes="href python:url(['/files', revid], file_id=file.file_id)"
+               tal:attributes="href python:url(['/files', file.change.revno], file_id=file.file_id)"
                tal:content="string:${file/filename}/">
               ${file.filename}/
             </a>
@@ -90,14 +90,14 @@
               ${file.filename}@
             </span>
             <a tal:condition="python:file.kind=='file'"
-               tal:attributes="href python:url(['/annotate', revid], file_id=file.file_id);
+               tal:attributes="href python:url(['/annotate', file.change.revno], file_id=file.file_id);
                                title string:Annotate ${file/filename}"
                tal:content="string:${file/filename}">
               ${file.filename}
             </a>
           </td>
           <td class="revision">
-            <a tal:attributes="href python:url(['/revision', file.revid], start_revid=file.revid, file_id=file.file_id);
+            <a tal:attributes="href python:url(['/revision', file.change.revno], filter_file_id=file.file_id);
                                title string:Show revision ${file/change/revno}"
                tal:content="python:util.trunc(file.change.revno, 15)">
               ${text}

=== modified file 'loggerhead/templates/modified-file-link-log.pt'
--- loggerhead/templates/modified-file-link-log.pt	2008-06-11 02:19:53 +0000
+++ loggerhead/templates/modified-file-link-log.pt	2008-06-12 03:08:14 +0000
@@ -1,5 +1,5 @@
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<a tal:attributes="href python:'javascript:diff_url(\'' + url(['/revision', entry.revid]) + '#' + item.filename + '\')';
+<a tal:attributes="href python:'javascript:diff_url(\'' + url(['/revision', entry.revno], clear=1) + '#' + item.filename + '\')';
                    title string:Annotate ${item/filename}"
    tal:content="item/filename" class="filename">
   item/filename

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2008-06-11 03:17:41 +0000
+++ loggerhead/templates/revision.pt	2008-06-13 05:00:06 +0000
@@ -125,7 +125,7 @@
                 <tr>
                   <th class="filename" colspan="4">
                     <tal:b content="structure python:collapse_button('file', u, 'table-row')" />
-                    <a tal:attributes="href  python:url(['/annotate', change.revid], file_id=item.file_id);
+                    <a tal:attributes="href  python:url(['/annotate', change.revno], clear=1, file_id=item.file_id);
                                        name  string:${item/filename}-s;
                                        title string:Annotate ${item/filename}"
                        tal:content="item/filename">

=== modified file 'loggerhead/templates/revisioninfo.pt'
--- loggerhead/templates/revisioninfo.pt	2008-05-29 05:58:01 +0000
+++ loggerhead/templates/revisioninfo.pt	2008-06-13 05:00:06 +0000
@@ -28,8 +28,7 @@
       <td class="children">
         <a title="Show history"
            tal:attributes="title string:Show revision ${merge_point/revno};
-                           href python:url(['/revision', merge_point.revid],
-                                           clear=1, start_revid=merge_point.revid)"
+                                        href python:url(['/revision', merge_point.revno])"
            tal:content="python:revno_with_nick(merge_point)">
           ${text}
         </a>
@@ -45,7 +44,7 @@
       <td class="parents">
         <a title="Show history"
            tal:attributes="title string:Show revision ${parent/revno};
-                             href python:url(['/revision', parent.revid], clear=1, start_revid=parent.revid)"
+                           href python:url(['/revision', parent.revno], clear=1)"
            tal:content="python:revno_with_nick(parent)">
           class="revlink">
           ${text}
@@ -65,6 +64,17 @@
       </td>
     </tr>
 
+    <tr>
+      <th class="revid">
+        revision id:
+      </th>
+      <td class="revid">
+          <span tal:content="python:change.revid">
+              revid
+        </span>
+      </td>
+    </tr>
+
     <tal:block condition="nocall:modified_file_link">
       <tr class="divider">
         <th>

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2008-06-16 09:41:35 +0000
+++ loggerhead/util.py	2008-06-13 05:44:04 +0000
@@ -37,13 +37,6 @@
 
 log = logging.getLogger("loggerhead.controllers")
 
-def fix_year(year):
-    if year < 70:
-        year += 2000
-    if year < 100:
-        year += 1900
-    return year
-
 # Display of times.
 
 # date_day -- just the day
@@ -356,17 +349,21 @@
 
     navigation.prev_page_revid = get_offset(-1 * navigation.pagesize)
     navigation.next_page_revid = get_offset(1 * navigation.pagesize)
+    prev_page_revno = navigation.branch._history.get_revno(
+            navigation.prev_page_revid)
+    next_page_revno = navigation.branch._history.get_revno(
+            navigation.next_page_revid)
 
     params = { 'filter_file_id': navigation.filter_file_id }
     if getattr(navigation, 'query', None) is not None:
         params['q'] = navigation.query
-    else:
-        params['start_revid'] = navigation.start_revid
 
     if navigation.prev_page_revid:
-        navigation.prev_page_url = navigation.branch.url([ navigation.scan_url, navigation.prev_page_revid ], **get_context(**params))
+        navigation.prev_page_url = navigation.branch.url([navigation.scan_url,
+            prev_page_revno], **params)
     if navigation.next_page_revid:
-        navigation.next_page_url = navigation.branch.url([ navigation.scan_url, navigation.next_page_revid ], **get_context(**params))
+        navigation.next_page_url = navigation.branch.url([navigation.scan_url,
+            next_page_revno], **params)
 
 
 def log_exception(log):
@@ -476,6 +473,9 @@
 
 def get_context(**overrides):
     """
+    Soon to be deprecated.
+
+
     return a context map that may be overriden by specific values passed in,
     but only contains keys from the list of valid context keys.
 

